@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/mman.h>
 #include <sys/prctl.h>
 #include <sys/ptrace.h>
 #include <sys/syscall.h>
@@ -35,8 +36,10 @@
 
 #include "mozilla/Array.h"
 #include "mozilla/Atomics.h"
+#include "mozilla/Attributes.h"
 #include "mozilla/Range.h"
 #include "mozilla/SandboxInfo.h"
+#include "mozilla/StackWalk.h"
 #include "mozilla/Span.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/Unused.h"
@@ -80,7 +83,7 @@ mozilla::Atomic<int> gSeccompTsyncBroadcastSignum(0);
 
 namespace mozilla {
 
-static bool gSandboxCrashOnError = false;
+static mozilla::Atomic<bool> gSandboxCrashOnError(false);
 
 // This is initialized by SandboxSetCrashFunc().
 SandboxCrashFunc gSandboxCrashFunc;
@@ -115,7 +118,8 @@ static bool ContextIsError(const ucontext_t* aContext, int aError) {
  * that it could be in async signal context (e.g., intercepting an
  * open() called from an async signal handler).
  */
-static void SigSysHandler(int nr, siginfo_t* info, void* void_context) {
+MOZ_NEVER_INLINE static void SigSysHandler(int nr, siginfo_t* info,
+                                           void* void_context) {
   ucontext_t* ctx = static_cast<ucontext_t*>(void_context);
   // This shouldn't ever be null, but the Chromium handler checks for
   // that and refrains from crashing, so let's not crash release builds:
@@ -148,7 +152,7 @@ static void SigSysHandler(int nr, siginfo_t* info, void* void_context) {
     // Bug 1017393: record syscall number somewhere useful.
     info->si_addr = reinterpret_cast<void*>(report.mSyscall);
 
-    gSandboxCrashFunc(nr, info, &savedCtx);
+    gSandboxCrashFunc(nr, info, &savedCtx, CallerPC());
     _exit(127);
   }
 }
@@ -211,8 +215,8 @@ static void InstallSigSysHandler(void) {
  * @see SandboxInfo
  * @see BroadcastSetThreadSandbox
  */
-static bool MOZ_MUST_USE InstallSyscallFilter(const sock_fprog* aProg,
-                                              bool aUseTSync) {
+[[nodiscard]] static bool InstallSyscallFilter(const sock_fprog* aProg,
+                                               bool aUseTSync) {
   if (prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0)) {
     if (!aUseTSync && errno == ETXTBSY) {
       return false;
@@ -226,14 +230,14 @@ static bool MOZ_MUST_USE InstallSyscallFilter(const sock_fprog* aProg,
                 SECCOMP_FILTER_FLAG_TSYNC, aProg) != 0) {
       SANDBOX_LOG_ERROR("thread-synchronized seccomp failed: %s",
                         strerror(errno));
-      MOZ_CRASH("prctl(PR_SET_SECCOMP, SECCOMP_MODE_FILTER)");
+      MOZ_CRASH("seccomp+tsync failed, but kernel supports tsync");
     }
   } else {
     if (prctl(PR_SET_SECCOMP, SECCOMP_MODE_FILTER, (unsigned long)aProg, 0,
               0)) {
       SANDBOX_LOG_ERROR("prctl(PR_SET_SECCOMP, SECCOMP_MODE_FILTER) failed: %s",
                         strerror(errno));
-      MOZ_CRASH("seccomp+tsync failed, but kernel supports tsync");
+      MOZ_CRASH("prctl(PR_SET_SECCOMP, SECCOMP_MODE_FILTER)");
     }
   }
   return true;
@@ -284,7 +288,8 @@ static void SetThreadSandboxHandler(int signum) {
 }
 
 static void EnterChroot() {
-  if (!PR_GetEnv(kSandboxChrootEnvFlag)) {
+  const char* env = PR_GetEnv(kSandboxChrootEnvFlag);
+  if (!env || !*env || *env == '0') {
     return;
   }
   char msg = kSandboxChrootRequest;
@@ -499,6 +504,14 @@ void SandboxEarlyInit() {
   }
 }
 
+static void RunGlibcLazyInitializers() {
+  // Make glibc's lazy initialization of shm_open() run before sandboxing
+  int fd = shm_open("/dummy", O_RDONLY, 0);
+  if (fd > 0) {
+    close(fd);  // In the unlikely case we actually opened something
+  }
+}
+
 static void SandboxLateInit() {
 #ifdef NIGHTLY_BUILD
   gSandboxCrashOnError = true;
@@ -515,6 +528,8 @@ static void SandboxLateInit() {
       gSandboxCrashOnError = envVar[0] != '0';
     }
   }
+
+  RunGlibcLazyInitializers();
 }
 
 // Common code for sandbox startup.
@@ -640,9 +655,11 @@ void SetMediaPluginSandbox(const char* aFilePath) {
   auto files = new SandboxOpenedFiles();
   files->Add(std::move(plugin));
   files->Add("/dev/urandom", true);
+  files->Add("/etc/ld.so.cache");  // Needed for NSS in clearkey.
   files->Add("/sys/devices/system/cpu/cpu0/tsc_freq_khz");
   files->Add("/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_max_freq");
   files->Add("/proc/cpuinfo");  // Info also available via CPUID instruction.
+  files->Add("/proc/sys/crypto/fips_enabled");  // Needed for NSS in clearkey.
 #ifdef __i386__
   files->Add("/proc/self/auxv");  // Info also in process's address space.
 #endif
@@ -672,4 +689,30 @@ void SetRemoteDataDecoderSandbox(int aBroker) {
   SetCurrentProcessSandbox(GetDecoderSandboxPolicy(sBroker));
 }
 
+void SetSocketProcessSandbox(int aBroker) {
+  if (!SandboxInfo::Get().Test(SandboxInfo::kHasSeccompBPF) ||
+      PR_GetEnv("MOZ_DISABLE_SOCKET_PROCESS_SANDBOX")) {
+    if (aBroker >= 0) {
+      close(aBroker);
+    }
+    return;
+  }
+
+  gSandboxReporterClient =
+      new SandboxReporterClient(SandboxReport::ProcType::SOCKET_PROCESS);
+
+  static SandboxBrokerClient* sBroker;
+  if (aBroker >= 0) {
+    sBroker = new SandboxBrokerClient(aBroker);
+  }
+
+  SetCurrentProcessSandbox(GetSocketProcessSandboxPolicy(sBroker));
+}
+
+bool SetSandboxCrashOnError(bool aValue) {
+  bool oldValue = gSandboxCrashOnError;
+  gSandboxCrashOnError = aValue;
+  return oldValue;
+}
+
 }  // namespace mozilla
