@@ -2,12 +2,16 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const { OS } = ChromeUtils.import("resource://gre/modules/osfile.jsm");
+const { ComponentUtils } = ChromeUtils.import(
+  "resource://gre/modules/ComponentUtils.jsm"
+);
 const { XPCOMUtils } = ChromeUtils.import(
   "resource://gre/modules/XPCOMUtils.jsm"
 );
 const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
+const TOPIC_PDFJS_HANDLER_CHANGED = "pdfjs:handlerChanged";
+
 ChromeUtils.defineModuleGetter(
   this,
   "FileUtils",
@@ -40,16 +44,19 @@ function HandlerService() {
 HandlerService.prototype = {
   classID: Components.ID("{220cc253-b60f-41f6-b9cf-fdcb325f970f}"),
   QueryInterface: ChromeUtils.generateQI([
-    Ci.nsISupportsWeakReference,
-    Ci.nsIHandlerService,
-    Ci.nsIObserver,
+    "nsISupportsWeakReference",
+    "nsIHandlerService",
+    "nsIObserver",
   ]),
 
   __store: null,
   get _store() {
     if (!this.__store) {
       this.__store = new JSONFile({
-        path: OS.Path.join(OS.Constants.Path.profileDir, "handlers.json"),
+        path: PathUtils.join(
+          Services.dirsvc.get("ProfD", Ci.nsIFile).path,
+          "handlers.json"
+        ),
         dataPostProcessor: this._dataPostProcessor.bind(this),
       });
     }
@@ -107,7 +114,7 @@ HandlerService.prototype = {
 
     try {
       prefsDefaultHandlersVersion = Number(prefsDefaultHandlersVersion.data);
-      let locale = Services.locale.appLocaleAsLangTag;
+      let locale = Services.locale.appLocaleAsBCP47;
 
       let defaultHandlersVersion =
         this._store.data.defaultHandlersVersion[locale] || 0;
@@ -116,6 +123,8 @@ HandlerService.prototype = {
         this._store.data.defaultHandlersVersion[
           locale
         ] = prefsDefaultHandlersVersion;
+        // Now save the result:
+        this._store.saveSoon();
       }
     } catch (ex) {
       Cu.reportError(ex);
@@ -151,22 +160,63 @@ HandlerService.prototype = {
       } catch (ex) {}
     }
 
-    for (let scheme of Object.keys(schemes)) {
-      let protoInfo = gExternalProtocolService.getProtocolHandlerInfo(scheme);
-
-      // cache the possible handlers to avoid extra xpconnect traversals.
-      let possibleHandlers = protoInfo.possibleApplicationHandlers;
-
-      for (let handlerNumber of Object.keys(schemes[scheme])) {
-        let handlerApp = this.handlerAppFromSerializable(
-          schemes[scheme][handlerNumber]
-        );
-        // If there is already a handler registered with the same template
-        // URL, the newly added one will be ignored when saving.
-        possibleHandlers.appendElement(handlerApp);
+    // Now drop any entries without a uriTemplate, or with a broken one.
+    // The Array.from calls ensure we can safely delete things without
+    // affecting the iterator.
+    for (let [scheme, handlerObject] of Array.from(Object.entries(schemes))) {
+      let handlers = Array.from(Object.entries(handlerObject));
+      let validHandlers = 0;
+      for (let [key, obj] of handlers) {
+        if (
+          !obj.uriTemplate ||
+          !obj.uriTemplate.startsWith("https://") ||
+          !obj.uriTemplate.toLowerCase().includes("%s")
+        ) {
+          delete handlerObject[key];
+        } else {
+          validHandlers++;
+        }
       }
+      if (!validHandlers) {
+        delete schemes[scheme];
+      }
+    }
 
-      this.store(protoInfo);
+    // Now, we're going to cheat. Terribly. The idiologically correct way
+    // of implementing the following bit of code would be to fetch the
+    // handler info objects from the protocol service, manipulate those,
+    // and then store each of them.
+    // However, that's expensive. It causes us to talk to the OS about
+    // default apps, which causes the OS to go hit the disk.
+    // All we're trying to do is insert some web apps into the list. We
+    // don't care what's already in the file, we just want to do the
+    // equivalent of appending into the database. So let's just go do that:
+    for (let scheme of Object.keys(schemes)) {
+      let existingSchemeInfo = this._store.data.schemes[scheme];
+      if (!existingSchemeInfo) {
+        // Haven't seen this scheme before. Default to asking which app the
+        // user wants to use:
+        existingSchemeInfo = {
+          // Signal to future readers that we didn't ask the OS anything.
+          // When the entry is first used, get the info from the OS.
+          stubEntry: true,
+          // The first item in the list is the preferred handler, and
+          // there isn't one, so we fill in null:
+          handlers: [null],
+        };
+        this._store.data.schemes[scheme] = existingSchemeInfo;
+      }
+      let { handlers } = existingSchemeInfo;
+      for (let handlerNumber of Object.keys(schemes[scheme])) {
+        let newHandler = schemes[scheme][handlerNumber];
+        // If there is already a handler registered with the same template
+        // URL, ignore the new one:
+        let matchingTemplate = handler =>
+          handler && handler.uriTemplate == newHandler.uriTemplate;
+        if (!handlers.some(matchingTemplate)) {
+          handlers.push(newHandler);
+        }
+      }
     }
   },
 
@@ -219,6 +269,64 @@ HandlerService.prototype = {
           }
         }
       },
+      // See https://bugzilla.mozilla.org/show_bug.cgi?id=1526890 for context.
+      "secure-mail": () => {
+        const kSubstitutions = new Map([
+          [
+            "http://compose.mail.yahoo.co.jp/ym/Compose?To=%s",
+            "https://mail.yahoo.co.jp/compose/?To=%s",
+          ],
+          [
+            "http://www.inbox.lv/rfc2368/?value=%s",
+            "https://mail.inbox.lv/compose?to=%s",
+          ],
+          [
+            "http://poczta.interia.pl/mh/?mailto=%s",
+            "https://poczta.interia.pl/mh/?mailto=%s",
+          ],
+          [
+            "http://win.mail.ru/cgi-bin/sentmsg?mailto=%s",
+            "https://e.mail.ru/cgi-bin/sentmsg?mailto=%s",
+          ],
+        ]);
+
+        function maybeReplaceURL(app) {
+          if (app instanceof Ci.nsIWebHandlerApp) {
+            let { uriTemplate } = app;
+            let sub = kSubstitutions.get(uriTemplate);
+            if (sub) {
+              app.uriTemplate = sub;
+              return true;
+            }
+          }
+          return false;
+        }
+        let mailHandler = gExternalProtocolService.getProtocolHandlerInfo(
+          "mailto"
+        );
+        if (this.exists(mailHandler)) {
+          this.fillHandlerInfo(mailHandler, "");
+          let handlers = mailHandler.possibleApplicationHandlers;
+          let shouldStore = false;
+          for (let i = handlers.length - 1; i >= 0; i--) {
+            let app = handlers.queryElementAt(i, Ci.nsIHandlerApp);
+            // Note: will evaluate the RHS because it's a binary rather than
+            // logical or.
+            shouldStore |= maybeReplaceURL(app);
+          }
+          // Then check the preferred handler.
+          if (mailHandler.preferredApplicationHandler) {
+            shouldStore |= maybeReplaceURL(
+              mailHandler.preferredApplicationHandler
+            );
+          }
+          // Then store, if we changed anything. Note that store() handles
+          // duplicates, so we don't have to.
+          if (shouldStore) {
+            this.store(mailHandler);
+          }
+        }
+      },
     };
     let migrationsToRun = Services.prefs.getCharPref(
       "browser.handlers.migrations",
@@ -264,7 +372,10 @@ HandlerService.prototype = {
   asyncInit() {
     if (!this.__store) {
       this.__store = new JSONFile({
-        path: OS.Path.join(OS.Constants.Path.profileDir, "handlers.json"),
+        path: PathUtils.join(
+          Services.dirsvc.get("ProfD", Ci.nsIFile).path,
+          "handlers.json"
+        ),
         dataPostProcessor: this._dataPostProcessor.bind(this),
       });
       this.__store
@@ -299,7 +410,7 @@ HandlerService.prototype = {
       //
       let handler = new Proxy(
         {
-          QueryInterface: ChromeUtils.generateQI([Ci.nsIHandlerInfo]),
+          QueryInterface: ChromeUtils.generateQI(["nsIHandlerInfo"]),
           type,
           get _handlerInfo() {
             delete this._handlerInfo;
@@ -338,7 +449,11 @@ HandlerService.prototype = {
     if (
       handlerInfo.preferredAction == Ci.nsIHandlerInfo.saveToDisk ||
       handlerInfo.preferredAction == Ci.nsIHandlerInfo.useSystemDefault ||
-      handlerInfo.preferredAction == Ci.nsIHandlerInfo.handleInternally
+      handlerInfo.preferredAction == Ci.nsIHandlerInfo.handleInternally ||
+      (handlerInfo.preferredAction == Ci.nsIHandlerInfo.alwaysAsk &&
+        Services.prefs.getBoolPref(
+          "browser.download.improvements_to_download_panel"
+        ))
     ) {
       storedHandlerInfo.action = handlerInfo.preferredAction;
     } else {
@@ -396,7 +511,16 @@ HandlerService.prototype = {
       }
     }
 
+    // If we're saving *anything*, it stops being a stub:
+    delete storedHandlerInfo.stubEntry;
+
     this._store.saveSoon();
+
+    // Now notify PDF.js. This is hacky, but a lot better than expecting all
+    // the consumers to do it...
+    if (handlerInfo.type == "application/pdf") {
+      Services.obs.notifyObservers(null, TOPIC_PDFJS_HANDLER_CHANGED);
+    }
   },
 
   // nsIHandlerService
@@ -412,22 +536,33 @@ HandlerService.prototype = {
       );
     }
 
-    handlerInfo.preferredAction = storedHandlerInfo.action;
-    handlerInfo.alwaysAskBeforeHandling = !!storedHandlerInfo.ask;
-
-    // If the first item is not null, it is also the preferred handler. Since
-    // we cannot modify the stored array, use a boolean to keep track of this.
-    let isFirstItem = true;
-    for (let handler of storedHandlerInfo.handlers || [null]) {
-      let handlerApp = this.handlerAppFromSerializable(handler || {});
-      if (isFirstItem) {
-        isFirstItem = false;
-        handlerInfo.preferredApplicationHandler = handlerApp;
-      }
-      if (handlerApp) {
-        handlerInfo.possibleApplicationHandlers.appendElement(handlerApp);
+    let isStub = !!storedHandlerInfo.stubEntry;
+    // In the normal case, this is not a stub, so we can just read stored info
+    // and write to the handlerInfo object we were passed.
+    if (!isStub) {
+      handlerInfo.preferredAction = storedHandlerInfo.action;
+      handlerInfo.alwaysAskBeforeHandling = !!storedHandlerInfo.ask;
+    } else {
+      // If we've got a stub, ensure the defaults are still set:
+      gExternalProtocolService.setProtocolHandlerDefaults(
+        handlerInfo,
+        handlerInfo.hasDefaultHandler
+      );
+      if (
+        handlerInfo.preferredAction == Ci.nsIHandlerInfo.alwaysAsk &&
+        handlerInfo.alwaysAskBeforeHandling
+      ) {
+        // `store` will default to `useHelperApp` because `alwaysAsk` is
+        // not one of the 3 recognized options; for compatibility, do
+        // the same here.
+        handlerInfo.preferredAction = Ci.nsIHandlerInfo.useHelperApp;
       }
     }
+    // If it *is* a stub, don't override alwaysAskBeforeHandling or the
+    // preferred actions. Instead, just append the stored handlers, without
+    // overriding the preferred app, and then schedule a task to store proper
+    // info for this handler.
+    this._appendStoredHandlers(handlerInfo, storedHandlerInfo.handlers, isStub);
 
     if (this._isMIMEInfo(handlerInfo) && storedHandlerInfo.extensions) {
       for (let extension of storedHandlerInfo.extensions) {
@@ -436,6 +571,34 @@ HandlerService.prototype = {
     }
   },
 
+  /**
+   * Private method to inject stored handler information into an nsIHandlerInfo
+   * instance.
+   * @param handlerInfo           the nsIHandlerInfo instance to write to
+   * @param storedHandlers        the stored handlers
+   * @param keepPreferredApp      whether to keep the handlerInfo's
+   *                              preferredApplicationHandler or override it
+   *                              (default: false, ie override it)
+   */
+  _appendStoredHandlers(handlerInfo, storedHandlers, keepPreferredApp) {
+    // If the first item is not null, it is also the preferred handler. Since
+    // we cannot modify the stored array, use a boolean to keep track of this.
+    let isFirstItem = true;
+    for (let handler of storedHandlers || [null]) {
+      let handlerApp = this.handlerAppFromSerializable(handler || {});
+      if (isFirstItem) {
+        isFirstItem = false;
+        // Do not overwrite the preferred app unless that's allowed
+        if (!keepPreferredApp) {
+          handlerInfo.preferredApplicationHandler = handlerApp;
+        }
+      }
+      if (handlerApp) {
+        handlerInfo.possibleApplicationHandlers.appendElement(handlerApp);
+      }
+    }
+  },
+
   /**
    * @param handler
    *        A nsIHandlerApp handler app
@@ -571,4 +734,4 @@ HandlerService.prototype = {
   },
 };
 
-this.NSGetFactory = XPCOMUtils.generateNSGetFactory([HandlerService]);
+this.NSGetFactory = ComponentUtils.generateNSGetFactory([HandlerService]);
