@@ -11,6 +11,10 @@
 #include "nsMIMEInfoUnix.h"
 #ifdef MOZ_WIDGET_GTK
 #  include "nsGNOMERegistry.h"
+#  ifdef MOZ_BUILD_APP_IS_BROWSER
+#    include "nsIToolkitShellService.h"
+#    include "nsIGNOMEShellService.h"
+#  endif
 #endif
 #include "nsISupports.h"
 #include "nsString.h"
@@ -26,6 +30,7 @@
 #include "nsCRT.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsDirectoryServiceUtils.h"
+#include "nsXULAppAPI.h"
 #include "ContentHandlerService.h"
 #include "prenv.h"  // for PR_GetEnv()
 #include "mozilla/Preferences.h"
@@ -222,10 +227,9 @@ nsresult nsOSHelperAppService::LookUpTypeAndDescription(
 inline bool IsNetscapeFormat(const nsACString& aBuffer) {
   return StringBeginsWith(
              aBuffer,
-             NS_LITERAL_CSTRING(
+             nsLiteralCString(
                  "#--Netscape Communications Corporation MIME Information")) ||
-         StringBeginsWith(aBuffer,
-                          NS_LITERAL_CSTRING("#--MCOM MIME Information"));
+         StringBeginsWith(aBuffer, "#--MCOM MIME Information"_ns);
 }
 
 /*
@@ -568,7 +572,7 @@ nsresult nsOSHelperAppService::ParseNetscapeMIMETypesEntry(
 
   // Get the major and minor types
   // First the major type
-  if (!FindInReadable(NS_LITERAL_STRING("type="), match_start, match_end)) {
+  if (!FindInReadable(u"type="_ns, match_start, match_end)) {
     return NS_ERROR_FAILURE;
   }
 
@@ -608,7 +612,7 @@ nsresult nsOSHelperAppService::ParseNetscapeMIMETypesEntry(
   // get the extensions
   match_start = match_end;
   match_end = end_iter;
-  if (FindInReadable(NS_LITERAL_STRING("exts="), match_start, match_end)) {
+  if (FindInReadable(u"exts="_ns, match_start, match_end)) {
     nsAString::const_iterator extStart, extEnd;
 
     if (match_end == end_iter ||
@@ -619,7 +623,7 @@ nsresult nsOSHelperAppService::ParseNetscapeMIMETypesEntry(
     extStart = match_end;
     match_start = extStart;
     match_end = end_iter;
-    if (FindInReadable(NS_LITERAL_STRING("desc=\""), match_start, match_end)) {
+    if (FindInReadable(u"desc=\""_ns, match_start, match_end)) {
       // exts= before desc=, so we have to find the actual end of the extensions
       extEnd = match_start;
       if (extEnd == extStart) {
@@ -646,11 +650,11 @@ nsresult nsOSHelperAppService::ParseNetscapeMIMETypesEntry(
   // get the description
   match_start = start_iter;
   match_end = end_iter;
-  if (FindInReadable(NS_LITERAL_STRING("desc=\""), match_start, match_end)) {
+  if (FindInReadable(u"desc=\""_ns, match_start, match_end)) {
     aDescriptionStart = match_end;
     match_start = aDescriptionStart;
     match_end = end_iter;
-    if (FindInReadable(NS_LITERAL_STRING("exts="), match_start, match_end)) {
+    if (FindInReadable(u"exts="_ns, match_start, match_end)) {
       // exts= after desc=, so have to find actual end of description
       aDescriptionEnd = match_start;
       if (aDescriptionEnd == aDescriptionStart) {
@@ -777,15 +781,13 @@ nsresult nsOSHelperAppService::LookUpHandlerAndDescription(
 
   // maybe we have an entry for "aMajorType/*"?
   if (NS_FAILED(rv)) {
-    rv = DoLookUpHandlerAndDescription(aMajorType, NS_LITERAL_STRING("*"),
-                                       aHandler, aDescription, aMozillaFlags,
-                                       true);
+    rv = DoLookUpHandlerAndDescription(aMajorType, u"*"_ns, aHandler,
+                                       aDescription, aMozillaFlags, true);
   }
 
   if (NS_FAILED(rv)) {
-    rv = DoLookUpHandlerAndDescription(aMajorType, NS_LITERAL_STRING("*"),
-                                       aHandler, aDescription, aMozillaFlags,
-                                       false);
+    rv = DoLookUpHandlerAndDescription(aMajorType, u"*"_ns, aHandler,
+                                       aDescription, aMozillaFlags, false);
   }
 
   return rv;
@@ -970,7 +972,7 @@ nsresult nsOSHelperAppService::GetHandlerAndDescriptionFromMailcapFile(
                   nsCOMPtr<nsIFile> file(
                       do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv));
                   if (NS_FAILED(rv)) continue;
-                  rv = file->InitWithNativePath(NS_LITERAL_CSTRING("/bin/sh"));
+                  rv = file->InitWithNativePath("/bin/sh"_ns);
                   if (NS_FAILED(rv)) continue;
                   rv = process->Init(file);
                   if (NS_FAILED(rv)) continue;
@@ -1057,6 +1059,12 @@ NS_IMETHODIMP nsOSHelperAppService::GetApplicationDescription(
 NS_IMETHODIMP nsOSHelperAppService::IsCurrentAppOSDefaultForProtocol(
     const nsACString& aScheme, bool* _retval) {
   *_retval = false;
+#if defined(MOZ_BUILD_APP_IS_BROWSER) && defined(MOZ_WIDGET_GTK)
+  if (nsCOMPtr<nsIGNOMEShellService> shell =
+          do_GetService(NS_TOOLKITSHELLSERVICE_CONTRACTID)) {
+    return shell->IsDefaultForScheme(aScheme, _retval);
+  }
+#endif
   return NS_OK;
 }
 
@@ -1171,8 +1179,7 @@ already_AddRefed<nsMIMEInfoBase> nsOSHelperAppService::GetFromExtension(
     return nullptr;
   }
 
-  nsAutoCString mimeType(asciiMajorType + NS_LITERAL_CSTRING("/") +
-                         asciiMinorType);
+  nsAutoCString mimeType(asciiMajorType + "/"_ns + asciiMinorType);
   RefPtr<nsMIMEInfoUnix> mimeInfo = new nsMIMEInfoUnix(mimeType);
 
   mimeInfo->AppendExtension(aFileExt);
@@ -1276,12 +1283,12 @@ already_AddRefed<nsMIMEInfoBase> nsOSHelperAppService::GetFromType(
   }
 
   if (handler.IsEmpty()) {
-    DoLookUpHandlerAndDescription(majorType, NS_LITERAL_STRING("*"), handler,
+    DoLookUpHandlerAndDescription(majorType, u"*"_ns, handler,
                                   mailcap_description, mozillaFlags, true);
   }
 
   if (handler.IsEmpty()) {
-    DoLookUpHandlerAndDescription(majorType, NS_LITERAL_STRING("*"), handler,
+    DoLookUpHandlerAndDescription(majorType, u"*"_ns, handler,
                                   mailcap_description, mozillaFlags, false);
   }
 
