@@ -13,8 +13,7 @@
 #include <shellapi.h>
 #include "nsIMutableArray.h"
 #include "nsTArray.h"
-#include "shlobj.h"
-#include "windows.h"
+#include <shlobj.h>
 #include "nsIWindowsRegKey.h"
 #include "nsUnicharUtils.h"
 #include "nsITextToSubURI.h"
@@ -96,30 +95,26 @@ nsMIMEInfoWin::LaunchWithFile(nsIFile* aFile) {
   if (mPreferredAction == useSystemDefault) {
     if (mDefaultApplication &&
         StaticPrefs::browser_pdf_launchDefaultEdgeAsApp()) {
-      // Since Edgium is the default PDF handler, if we're using the OS default
-      // and it's Edgium prefer it's app mode so it operates as a PDF viewer
-      // (without browser toolbars). Bug 1632277.
-      bool isPdf;
-      rv = IsPdf(&isPdf);
-      if (NS_SUCCEEDED(rv) && isPdf) {
-        nsAutoCString defaultAppExecutable;
-        rv = mDefaultApplication->GetNativeLeafName(defaultAppExecutable);
-        if (NS_SUCCEEDED(rv) &&
-            defaultAppExecutable.LowerCaseEqualsLiteral("msedge.exe")) {
-          nsAutoString path;
-          rv = aFile->GetPath(path);
-          if (NS_SUCCEEDED(rv)) {
-            // If the --app flag doesn't work we'll want to fallback to a
-            // regular path. Send two args so we call `msedge.exe --app={path}
-            // {path}`.
-            nsAutoString appArg;
-            appArg.AppendLiteral("--app=");
-            appArg.Append(path);
-            const wchar_t* argv[] = {appArg.get(), path.get()};
+      // Since Edgium is the default handler for PDF and other kinds of files,
+      // if we're using the OS default and it's Edgium prefer its app mode so it
+      // operates as a viewer (without browser toolbars). Bug 1632277.
+      nsAutoCString defaultAppExecutable;
+      rv = mDefaultApplication->GetNativeLeafName(defaultAppExecutable);
+      if (NS_SUCCEEDED(rv) &&
+          defaultAppExecutable.LowerCaseEqualsLiteral("msedge.exe")) {
+        nsAutoString path;
+        rv = aFile->GetPath(path);
+        if (NS_SUCCEEDED(rv)) {
+          // If the --app flag doesn't work we'll want to fallback to a
+          // regular path. Send two args so we call `msedge.exe --app={path}
+          // {path}`.
+          nsAutoString appArg;
+          appArg.AppendLiteral("--app=");
+          appArg.Append(path);
+          const wchar_t* argv[] = {appArg.get(), path.get()};
 
-            return ShellExecuteWithIFile(mDefaultApplication,
-                                         mozilla::ArrayLength(argv), argv);
-          }
+          return ShellExecuteWithIFile(mDefaultApplication,
+                                       mozilla::ArrayLength(argv), argv);
         }
       }
     }
@@ -224,12 +219,10 @@ nsMIMEInfoWin::GetEnumerator(nsISimpleEnumerator** _retval) {
   nsCOMArray<nsIVariant> properties;
 
   nsCOMPtr<nsIVariant> variant;
-  GetProperty(NS_LITERAL_STRING("defaultApplicationIconURL"),
-              getter_AddRefs(variant));
+  GetProperty(u"defaultApplicationIconURL"_ns, getter_AddRefs(variant));
   if (variant) properties.AppendObject(variant);
 
-  GetProperty(NS_LITERAL_STRING("customApplicationIconURL"),
-              getter_AddRefs(variant));
+  GetProperty(u"customApplicationIconURL"_ns, getter_AddRefs(variant));
   if (variant) properties.AppendObject(variant);
 
   return NS_NewArrayEnumerator(_retval, properties, NS_GET_IID(nsIVariant));
@@ -295,8 +288,8 @@ nsresult nsMIMEInfoWin::LoadUriInternal(nsIURI* aURL) {
         do_GetService(NS_ITEXTTOSUBURI_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    if (NS_FAILED(textToSubURI->UnEscapeNonAsciiURI(NS_LITERAL_CSTRING("UTF-8"),
-                                                    urlSpec, utf16Spec))) {
+    if (NS_FAILED(textToSubURI->UnEscapeNonAsciiURI("UTF-8"_ns, urlSpec,
+                                                    utf16Spec))) {
       CopyASCIItoUTF16(urlSpec, utf16Spec);
     }
 
@@ -386,14 +379,12 @@ bool nsMIMEInfoWin::GetAppsVerbCommandHandler(const nsAString& appExeName,
 
   // Check for the NoOpenWith flag, if it exists
   uint32_t value;
-  if (NS_SUCCEEDED(
-          appKey->ReadIntValue(NS_LITERAL_STRING("NoOpenWith"), &value)) &&
+  if (NS_SUCCEEDED(appKey->ReadIntValue(u"NoOpenWith"_ns, &value)) &&
       value == 1)
     return false;
 
   nsAutoString dummy;
-  if (NS_SUCCEEDED(
-          appKey->ReadStringValue(NS_LITERAL_STRING("NoOpenWith"), dummy)))
+  if (NS_SUCCEEDED(appKey->ReadStringValue(u"NoOpenWith"_ns, dummy)))
     return false;
 
   appKey->Close();
@@ -411,8 +402,7 @@ bool nsMIMEInfoWin::GetAppsVerbCommandHandler(const nsAString& appExeName,
   if (NS_FAILED(rv)) return false;
 
   nsAutoString appFilesystemCommand;
-  if (NS_SUCCEEDED(
-          appKey->ReadStringValue(EmptyString(), appFilesystemCommand))) {
+  if (NS_SUCCEEDED(appKey->ReadStringValue(u""_ns, appFilesystemCommand))) {
     // Expand environment vars, clean up any misc.
     if (!nsLocalFile::CleanupCmdHandlerPath(appFilesystemCommand)) return false;
 
@@ -449,12 +439,11 @@ bool nsMIMEInfoWin::GetDllLaunchInfo(nsIFile* aDll, nsIFile* aFile,
 
   // Check for the NoOpenWith flag, if it exists
   uint32_t value;
-  rv = appKey->ReadIntValue(NS_LITERAL_STRING("NoOpenWith"), &value);
+  rv = appKey->ReadIntValue(u"NoOpenWith"_ns, &value);
   if (NS_SUCCEEDED(rv) && value == 1) return false;
 
   nsAutoString dummy;
-  if (NS_SUCCEEDED(
-          appKey->ReadStringValue(NS_LITERAL_STRING("NoOpenWith"), dummy)))
+  if (NS_SUCCEEDED(appKey->ReadStringValue(u"NoOpenWith"_ns, dummy)))
     return false;
 
   appKey->Close();
@@ -472,8 +461,7 @@ bool nsMIMEInfoWin::GetDllLaunchInfo(nsIFile* aDll, nsIFile* aFile,
   if (NS_FAILED(rv)) return false;
 
   nsAutoString appFilesystemCommand;
-  if (NS_SUCCEEDED(
-          appKey->ReadStringValue(EmptyString(), appFilesystemCommand))) {
+  if (NS_SUCCEEDED(appKey->ReadStringValue(u""_ns, appFilesystemCommand))) {
     // Replace embedded environment variables.
     uint32_t bufLength =
         ::ExpandEnvironmentStringsW(appFilesystemCommand.get(), nullptr, 0);
@@ -491,7 +479,7 @@ bool nsMIMEInfoWin::GetDllLaunchInfo(nsIFile* aDll, nsIFile* aFile,
     // C:\Windows\System32\rundll32.exe "C:\Program Files\Windows
     // Photo Gallery\PhotoViewer.dll", ImageView_Fullscreen %1
     nsAutoString params;
-    NS_NAMED_LITERAL_STRING(rundllSegment, "rundll32.exe ");
+    constexpr auto rundllSegment = u"rundll32.exe "_ns;
     int32_t index = appFilesystemCommand.Find(rundllSegment);
     if (index > kNotFound) {
       params.Append(
@@ -501,7 +489,7 @@ bool nsMIMEInfoWin::GetDllLaunchInfo(nsIFile* aDll, nsIFile* aFile,
     }
 
     // check to make sure we have a %1 and fill it
-    NS_NAMED_LITERAL_STRING(percentOneParam, "%1");
+    constexpr auto percentOneParam = u"%1"_ns;
     index = params.Find(percentOneParam);
     if (index == kNotFound)  // no parameter
       return false;
@@ -539,8 +527,7 @@ bool nsMIMEInfoWin::GetProgIDVerbCommandHandler(const nsAString& appProgIDName,
   if (NS_FAILED(rv)) return false;
 
   nsAutoString appFilesystemCommand;
-  if (NS_SUCCEEDED(
-          appKey->ReadStringValue(EmptyString(), appFilesystemCommand))) {
+  if (NS_SUCCEEDED(appKey->ReadStringValue(u""_ns, appFilesystemCommand))) {
     // Expand environment vars, clean up any misc.
     if (!nsLocalFile::CleanupCmdHandlerPath(appFilesystemCommand)) return false;
 
@@ -640,7 +627,7 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
                         nsIWindowsRegKey::ACCESS_QUERY_VALUE);
       if (NS_SUCCEEDED(rv)) {
         nsAutoString mimeFileExt;
-        if (NS_SUCCEEDED(regKey->ReadStringValue(EmptyString(), mimeFileExt))) {
+        if (NS_SUCCEEDED(regKey->ReadStringValue(u""_ns, mimeFileExt))) {
           CopyUTF16toUTF8(mimeFileExt, fileExt);
           extKnown = false;
         }
@@ -666,9 +653,9 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
                      workingRegistryPath, nsIWindowsRegKey::ACCESS_QUERY_VALUE);
     if (NS_SUCCEEDED(rv)) {
       nsAutoString appProgId;
-      if (NS_SUCCEEDED(regKey->ReadStringValue(EmptyString(), appProgId))) {
+      if (NS_SUCCEEDED(regKey->ReadStringValue(u""_ns, appProgId))) {
         // Bug 358297 - ignore the embedded internet explorer handler
-        if (appProgId != NS_LITERAL_STRING("XPSViewer.Document")) {
+        if (appProgId != u"XPSViewer.Document"_ns) {
           nsAutoString appFilesystemCommand;
           if (GetProgIDVerbCommandHandler(appProgId, appFilesystemCommand,
                                           false) &&
@@ -739,8 +726,8 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
 
     // HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion
     // \Explorer\FileExts\.ext\OpenWithList
-    workingRegistryPath = NS_LITERAL_STRING(
-        "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FileExts\\");
+    workingRegistryPath = nsLiteralString(
+        u"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FileExts\\");
     workingRegistryPath += fileExtToUse;
     workingRegistryPath.AppendLiteral("\\OpenWithList");
 
@@ -772,8 +759,8 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
 
     // HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion
     // \Explorer\FileExts\.ext\OpenWithProgids
-    workingRegistryPath = NS_LITERAL_STRING(
-        "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FileExts\\");
+    workingRegistryPath = nsLiteralString(
+        u"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FileExts\\");
     workingRegistryPath += fileExtToUse;
     workingRegistryPath.AppendLiteral("\\OpenWithProgids");
 
@@ -807,11 +794,9 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
                  nsIWindowsRegKey::ACCESS_QUERY_VALUE);
     if (NS_SUCCEEDED(rv)) {
       nsAutoString perceivedType;
-      rv = regKey->ReadStringValue(NS_LITERAL_STRING("PerceivedType"),
-                                   perceivedType);
+      rv = regKey->ReadStringValue(u"PerceivedType"_ns, perceivedType);
       if (NS_SUCCEEDED(rv)) {
-        nsAutoString openWithListPath(
-            NS_LITERAL_STRING("SystemFileAssociations\\"));
+        nsAutoString openWithListPath(u"SystemFileAssociations\\"_ns);
         openWithListPath.Append(perceivedType);  // no period
         openWithListPath.AppendLiteral("\\OpenWithList");
 
@@ -843,7 +828,7 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
   // Listing general purpose handlers, not specific to a mime type or file
   // extension
 
-  workingRegistryPath = NS_LITERAL_STRING("*\\OpenWithList");
+  workingRegistryPath = u"*\\OpenWithList"_ns;
 
   rv = regKey->Open(nsIWindowsRegKey::ROOT_KEY_CLASSES_ROOT,
                     workingRegistryPath, nsIWindowsRegKey::ACCESS_QUERY_VALUE);
@@ -866,7 +851,7 @@ nsMIMEInfoWin::GetPossibleLocalHandlers(nsIArray** _retval) {
   }
 
   // 8) General application's list - not file extension specific on windows
-  workingRegistryPath = NS_LITERAL_STRING("Applications");
+  workingRegistryPath = u"Applications"_ns;
 
   rv =
       regKey->Open(nsIWindowsRegKey::ROOT_KEY_CLASSES_ROOT, workingRegistryPath,
