@@ -36,10 +36,18 @@ using mozilla::LogLevel;
 
 static mozilla::LazyLogModule gWin32ClipboardLog("nsClipboard");
 
-// oddly, this isn't in the MSVC headers anywhere.
-UINT nsClipboard::CF_HTML = ::RegisterClipboardFormatW(L"HTML Format");
-UINT nsClipboard::CF_CUSTOMTYPES =
-    ::RegisterClipboardFormatW(L"application/x-moz-custom-clipdata");
+/* static */
+UINT nsClipboard::GetHtmlClipboardFormat() {
+  static UINT format = ::RegisterClipboardFormatW(L"HTML Format");
+  return format;
+}
+
+/* static */
+UINT nsClipboard::GetCustomClipboardFormat() {
+  static UINT format =
+      ::RegisterClipboardFormatW(L"application/x-moz-custom-clipdata");
+  return format;
+}
 
 //-------------------------------------------------------------------------
 //
@@ -96,9 +104,9 @@ UINT nsClipboard::GetFormat(const char* aMimeStr, bool aMapHTMLMime) {
     format = CF_HDROP;
   } else if ((strcmp(aMimeStr, kNativeHTMLMime) == 0) ||
              (aMapHTMLMime && strcmp(aMimeStr, kHTMLMime) == 0)) {
-    format = CF_HTML;
+    format = GetHtmlClipboardFormat();
   } else if (strcmp(aMimeStr, kCustomTypesMime) == 0) {
-    format = CF_CUSTOMTYPES;
+    format = GetCustomClipboardFormat();
   } else {
     format = ::RegisterClipboardFormatW(NS_ConvertASCIItoUTF16(aMimeStr).get());
   }
@@ -180,7 +188,8 @@ nsresult nsClipboard::SetupNativeDataObject(nsITransferable* aTransferable,
       // if we find text/html, also advertise win32's html flavor (which we will
       // convert on our own in nsDataObj::GetText().
       FORMATETC htmlFE;
-      SET_FORMATETC(htmlFE, CF_HTML, 0, DVASPECT_CONTENT, -1, TYMED_HGLOBAL);
+      SET_FORMATETC(htmlFE, GetHtmlClipboardFormat(), 0, DVASPECT_CONTENT, -1,
+                    TYMED_HGLOBAL);
       dObj->AddDataFlavor(kHTMLMime, &htmlFE);
     } else if (flavorStr.EqualsLiteral(kURLMime)) {
       // if we're a url, in addition to also being text, we need to register
@@ -470,7 +479,7 @@ nsresult nsClipboard::GetNativeDataOffClipboard(IDataObject* aDataObject,
                     do_CreateInstance("@mozilla.org/image/tools;1");
                 result = imgTools->DecodeImageFromBuffer(
                     clipboardData, allocLen,
-                    NS_LITERAL_CSTRING(IMAGE_BMP_MS_CLIPBOARD),
+                    nsLiteralCString(IMAGE_BMP_MS_CLIPBOARD),
                     getter_AddRefs(container));
                 if (NS_FAILED(result)) {
                   break;
@@ -484,9 +493,8 @@ nsresult nsClipboard::GetNativeDataOffClipboard(IDataObject* aDataObject,
                 }
 
                 nsCOMPtr<nsIInputStream> inputStream;
-                result =
-                    imgTools->EncodeImage(container, mimeType, EmptyString(),
-                                          getter_AddRefs(inputStream));
+                result = imgTools->EncodeImage(container, mimeType, u""_ns,
+                                               getter_AddRefs(inputStream));
                 if (NS_FAILED(result)) {
                   break;
                 }
@@ -550,14 +558,14 @@ nsresult nsClipboard::GetNativeDataOffClipboard(IDataObject* aDataObject,
               //        assumption. Stay tuned.
               uint32_t allocLen = 0;
               if (NS_SUCCEEDED(GetGlobalData(stm.hGlobal, aData, &allocLen))) {
-                if (fe.cfFormat == CF_HTML) {
+                if (fe.cfFormat == GetHtmlClipboardFormat()) {
                   // CF_HTML is actually UTF8, not unicode, so disregard the
                   // assumption above. We have to check the header for the
                   // actual length, and we'll do that in FindPlatformHTML().
                   // For now, return the allocLen. This case is mostly to
                   // ensure we don't try to call strlen on the buffer.
                   *aLen = allocLen;
-                } else if (fe.cfFormat == CF_CUSTOMTYPES) {
+                } else if (fe.cfFormat == GetCustomClipboardFormat()) {
                   // Binary data
                   *aLen = allocLen;
                 } else if (fe.cfFormat == preferredDropEffect) {
@@ -880,7 +888,7 @@ bool nsClipboard ::FindURLFromLocalFile(IDataObject* inDataObject, UINT inIndex,
         if (title.IsEmpty()) {
           title = urlString;
         }
-        *outData = ToNewUnicode(urlString + NS_LITERAL_STRING("\n") + title);
+        *outData = ToNewUnicode(urlString + u"\n"_ns + title);
         *outDataLen =
             NS_strlen(static_cast<char16_t*>(*outData)) * sizeof(char16_t);
 
@@ -925,7 +933,7 @@ bool nsClipboard ::FindURLFromNativeURL(IDataObject* inDataObject, UINT inIndex,
     // the internal mozilla URL format, text/x-moz-url, contains
     // URL\ntitle.  Since we don't actually have a title here,
     // just repeat the URL to fake it.
-    *outData = ToNewUnicode(urlString + NS_LITERAL_STRING("\n") + urlString);
+    *outData = ToNewUnicode(urlString + u"\n"_ns + urlString);
     *outDataLen =
         NS_strlen(static_cast<char16_t*>(*outData)) * sizeof(char16_t);
     free(tempOutData);
@@ -954,7 +962,7 @@ bool nsClipboard ::FindURLFromNativeURL(IDataObject* inDataObject, UINT inIndex,
       // the internal mozilla URL format, text/x-moz-url, contains
       // URL\ntitle.  Since we don't actually have a title here,
       // just repeat the URL to fake it.
-      *outData = ToNewUnicode(urlString + NS_LITERAL_STRING("\n") + urlString);
+      *outData = ToNewUnicode(urlString + u"\n"_ns + urlString);
       *outDataLen =
           NS_strlen(static_cast<char16_t*>(*outData)) * sizeof(char16_t);
       free(tempOutData);
@@ -990,7 +998,7 @@ void nsClipboard ::ResolveShortcut(nsIFile* aFile, nsACString& outURL) {
 // A file is an Internet Shortcut if it ends with .URL
 //
 bool nsClipboard ::IsInternetShortcut(const nsAString& inFileName) {
-  return StringEndsWith(inFileName, NS_LITERAL_STRING(".url"),
+  return StringEndsWith(inFileName, u".url"_ns,
                         nsCaseInsensitiveStringComparator);
 }  // IsInternetShortcut
 
