@@ -13,6 +13,7 @@
 #include "nsArrayUtils.h"
 #include "nsClipboard.h"
 #include "nsReadableUtils.h"
+#include "nsICookieJarSettings.h"
 #include "nsITransferable.h"
 #include "nsISupportsPrimitives.h"
 #include "IEnumFE.h"
@@ -24,7 +25,8 @@
 #include "nsEscape.h"
 #include "nsIURL.h"
 #include "nsNetUtil.h"
-#include "mozilla/Services.h"
+#include "mozilla/Components.h"
+#include "mozilla/SpinEventLoopUntil.h"
 #include "mozilla/Unused.h"
 #include "nsIOutputStream.h"
 #include "nscore.h"
@@ -68,17 +70,17 @@ nsDataObj::CStream::~CStream() {}
 //-----------------------------------------------------------------------------
 // helper - initializes the stream
 nsresult nsDataObj::CStream::Init(nsIURI* pSourceURI,
-                                  uint32_t aContentPolicyType,
-                                  nsIPrincipal* aRequestingPrincipal) {
+                                  nsContentPolicyType aContentPolicyType,
+                                  nsIPrincipal* aRequestingPrincipal,
+                                  nsICookieJarSettings* aCookieJarSettings) {
   // we can not create a channel without a requestingPrincipal
   if (!aRequestingPrincipal) {
     return NS_ERROR_FAILURE;
   }
   nsresult rv;
   rv = NS_NewChannel(getter_AddRefs(mChannel), pSourceURI, aRequestingPrincipal,
-                     nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_INHERITS,
-                     aContentPolicyType,
-                     nullptr,  // nsICookieJarSettings
+                     nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_INHERITS_SEC_CONTEXT,
+                     aContentPolicyType, aCookieJarSettings,
                      nullptr,  // PerformanceStorage
                      nullptr,  // loadGroup
                      nullptr,  // aCallbacks
@@ -312,8 +314,14 @@ HRESULT nsDataObj::CreateStream(IStream** outStream) {
       mTransferable->GetRequestingPrincipal();
   MOZ_ASSERT(requestingPrincipal, "can not create channel without a principal");
 
-  uint32_t contentPolicyType = mTransferable->GetContentPolicyType();
-  rv = pStream->Init(sourceURI, contentPolicyType, requestingPrincipal);
+  // Note that the cookieJarSettings could be null if the data object is for the
+  // image copy. We will fix this in Bug 1690532.
+  nsCOMPtr<nsICookieJarSettings> cookieJarSettings =
+      mTransferable->GetCookieJarSettings();
+
+  nsContentPolicyType contentPolicyType = mTransferable->GetContentPolicyType();
+  rv = pStream->Init(sourceURI, contentPolicyType, requestingPrincipal,
+                     cookieJarSettings);
   if (NS_FAILED(rv)) {
     pStream->Release();
     return E_FAIL;
@@ -430,14 +438,11 @@ STDMETHODIMP nsDataObj::CMemStream::Stat(STATSTG* statstg, DWORD dwFlags) {
   memset((void*)statstg, 0, sizeof(STATSTG));
 
   if (dwFlags != STATFLAG_NONAME) {
-    const nsString& wideFileName = EmptyString();
-
-    uint32_t nMaxNameLength = (wideFileName.Length() * 2) + 2;
-    void* retBuf = CoTaskMemAlloc(nMaxNameLength);  // freed by caller
+    constexpr size_t kMaxNameLength = sizeof(wchar_t);
+    void* retBuf = CoTaskMemAlloc(kMaxNameLength);  // freed by caller
     if (!retBuf) return STG_E_INSUFFICIENTMEMORY;
 
-    ZeroMemory(retBuf, nMaxNameLength);
-    memcpy(retBuf, wideFileName.get(), wideFileName.Length() * 2);
+    ZeroMemory(retBuf, kMaxNameLength);
     statstg->pwcsName = (LPOLESTR)retBuf;
   }
 
@@ -477,8 +482,7 @@ nsDataObj::nsDataObj(nsIURI* uri)
       mTransferable(nullptr),
       mIsAsyncMode(FALSE),
       mIsInOperation(FALSE) {
-  mIOThread = new LazyIdleThread(DEFAULT_THREAD_TIMEOUT_MS,
-                                 NS_LITERAL_CSTRING("nsDataObj"),
+  mIOThread = new LazyIdleThread(DEFAULT_THREAD_TIMEOUT_MS, "nsDataObj"_ns,
                                  LazyIdleThread::ManualShutdown);
   m_enumFE = new CEnumFormatEtc();
   m_enumFE->AddRef();
@@ -958,7 +962,7 @@ nsDataObj::GetDib(const nsACString& inFlavor, FORMATETC& aFormat,
   nsCOMPtr<imgITools> imgTools =
       do_CreateInstance("@mozilla.org/image/tools;1");
 
-  nsAutoString options(NS_LITERAL_STRING("bpp=32;"));
+  nsAutoString options(u"bpp=32;"_ns);
   if (aFormat.cfFormat == CF_DIBV5) {
     options.AppendLiteral("version=5");
   } else {
@@ -966,7 +970,7 @@ nsDataObj::GetDib(const nsACString& inFlavor, FORMATETC& aFormat,
   }
 
   nsCOMPtr<nsIInputStream> inputStream;
-  nsresult rv = imgTools->EncodeImage(image, NS_LITERAL_CSTRING(IMAGE_BMP),
+  nsresult rv = imgTools->EncodeImage(image, nsLiteralCString(IMAGE_BMP),
                                       options, getter_AddRefs(inputStream));
   if (NS_FAILED(rv) || !inputStream) {
     return E_FAIL;
@@ -1141,7 +1145,7 @@ static bool CreateFilenameFromTextW(nsString& aText, const wchar_t* aExtension,
 
 static bool GetLocalizedString(const char* aName, nsAString& aString) {
   nsCOMPtr<nsIStringBundleService> stringService =
-      mozilla::services::GetStringBundleService();
+      mozilla::components::StringBundle::Service();
   if (!stringService) return false;
 
   nsCOMPtr<nsIStringBundle> stringBundle;
@@ -1486,7 +1490,7 @@ HRESULT nsDataObj::GetText(const nsACString& aDataFlavor, FORMATETC& aFE,
       NS_WARNING("Oh no, couldn't convert unicode to plain text");
       return S_OK;
     }
-  } else if (aFE.cfFormat == nsClipboard::CF_HTML) {
+  } else if (aFE.cfFormat == nsClipboard::GetHtmlClipboardFormat()) {
     // Someone is asking for win32's HTML flavor. Convert our html fragment
     // from unicode to UTF-8 then put it into a format specified by msft.
     NS_ConvertUTF16toUTF8 converter(reinterpret_cast<char16_t*>(data));
@@ -1503,7 +1507,7 @@ HRESULT nsDataObj::GetText(const nsACString& aDataFlavor, FORMATETC& aFE,
       NS_WARNING("Oh no, couldn't convert to HTML");
       return S_OK;
     }
-  } else if (aFE.cfFormat != nsClipboard::CF_CUSTOMTYPES) {
+  } else if (aFE.cfFormat != nsClipboard::GetCustomClipboardFormat()) {
     // we assume that any data that isn't caught above is unicode. This may
     // be an erroneous assumption, but is true so far.
     allocLen += sizeof(char16_t);
@@ -1612,8 +1616,8 @@ HRESULT nsDataObj::DropImage(FORMATETC& aFE, STGMEDIUM& aSTG) {
     nsCOMPtr<imgITools> imgTools =
         do_CreateInstance("@mozilla.org/image/tools;1");
     nsCOMPtr<nsIInputStream> inputStream;
-    rv = imgTools->EncodeImage(image, NS_LITERAL_CSTRING(IMAGE_BMP),
-                               NS_LITERAL_STRING("bpp=32;version=3"),
+    rv = imgTools->EncodeImage(image, nsLiteralCString(IMAGE_BMP),
+                               u"bpp=32;version=3"_ns,
                                getter_AddRefs(inputStream));
     if (NS_FAILED(rv) || !inputStream) {
       return E_FAIL;
@@ -1974,9 +1978,9 @@ nsresult nsDataObj ::BuildPlatformHTML(const char* inOurHTML,
       (kSourceURLLength > 0 ? strlen(startSourceURLPrefix) : 0) +
       kSourceURLLength + (4 * kNumberLength);
 
-  NS_NAMED_LITERAL_CSTRING(htmlHeaderString, "<html><body>\r\n");
+  constexpr auto htmlHeaderString = "<html><body>\r\n"_ns;
 
-  NS_NAMED_LITERAL_CSTRING(fragmentHeaderString, "<!--StartFragment-->");
+  constexpr auto fragmentHeaderString = "<!--StartFragment-->"_ns;
 
   nsDependentCString trailingString(
       "<!--EndFragment-->\r\n"
