@@ -15,7 +15,7 @@
 #include "CompositorWidget.h"
 #include "nsWindowBase.h"
 #include "nsdefs.h"
-#include "nsIdleService.h"
+#include "nsUserIdleService.h"
 #include "nsToolkit.h"
 #include "nsString.h"
 #include "nsTArray.h"
@@ -42,11 +42,11 @@
 
 #ifdef ACCESSIBILITY
 #  include "oleacc.h"
-#  include "mozilla/a11y/Accessible.h"
+#  include "mozilla/a11y/LocalAccessible.h"
 #endif
 
 #include "nsUXThemeData.h"
-#include "nsIIdleServiceInternal.h"
+#include "nsIUserIdleServiceInternal.h"
 
 #include "IMMHandler.h"
 
@@ -126,8 +126,6 @@ class nsWindow final : public nsWindowBase {
   virtual nsWindowBase* GetParentWindowBase(bool aIncludeOwner) override;
   virtual bool IsTopLevelWidget() override { return mIsTopWidgetWindow; }
 
-  using nsWindowBase::DispatchPluginEvent;
-
   // nsIWidget interface
   using nsWindowBase::Create;  // for Create signature not overridden here
   [[nodiscard]] virtual nsresult Create(
@@ -180,8 +178,7 @@ class nsWindow final : public nsWindowBase {
   virtual LayoutDeviceIntRect GetClientBounds() override;
   virtual LayoutDeviceIntPoint GetClientOffset() override;
   void SetBackgroundColor(const nscolor& aColor) override;
-  virtual void SetCursor(nsCursor aDefaultCursor, imgIContainer* aCursorImage,
-                         uint32_t aHotspotX, uint32_t aHotspotY) override;
+  virtual void SetCursor(const Cursor&) override;
   virtual nsresult ConfigureChildren(
       const nsTArray<Configuration>& aConfigurations) override;
   virtual bool PrepareForFullscreenTransition(nsISupports** aData) override;
@@ -224,14 +221,16 @@ class nsWindow final : public nsWindowBase {
       int32_t aNativeKeyboardLayout, int32_t aNativeKeyCode,
       uint32_t aModifierFlags, const nsAString& aCharacters,
       const nsAString& aUnmodifiedCharacters, nsIObserver* aObserver) override;
-  virtual nsresult SynthesizeNativeMouseEvent(LayoutDeviceIntPoint aPoint,
-                                              uint32_t aNativeMessage,
-                                              uint32_t aModifierFlags,
-                                              nsIObserver* aObserver) override;
+  virtual nsresult SynthesizeNativeMouseEvent(
+      LayoutDeviceIntPoint aPoint, NativeMouseMessage aNativeMessage,
+      mozilla::MouseButton aButton, nsIWidget::Modifiers aModifierFlags,
+      nsIObserver* aObserver) override;
 
   virtual nsresult SynthesizeNativeMouseMove(LayoutDeviceIntPoint aPoint,
                                              nsIObserver* aObserver) override {
-    return SynthesizeNativeMouseEvent(aPoint, MOUSEEVENTF_MOVE, 0, aObserver);
+    return SynthesizeNativeMouseEvent(
+        aPoint, NativeMouseMessage::Move, mozilla::MouseButton::eNotPressed,
+        nsIWidget::Modifiers::NO_MODIFIERS, aObserver);
   }
 
   virtual nsresult SynthesizeNativeMouseScrollEvent(
@@ -263,24 +262,21 @@ class nsWindow final : public nsWindowBase {
   /**
    * Event helpers
    */
-  virtual bool DispatchMouseEvent(
-      mozilla::EventMessage aEventMessage, WPARAM wParam, LPARAM lParam,
-      bool aIsContextMenuKey = false,
-      int16_t aButton = mozilla::MouseButton::eLeft,
-      uint16_t aInputSource =
-          mozilla::dom::MouseEvent_Binding::MOZ_SOURCE_MOUSE,
-      WinPointerInfo* aPointerInfo = nullptr);
+  virtual bool DispatchMouseEvent(mozilla::EventMessage aEventMessage,
+                                  WPARAM wParam, LPARAM lParam,
+                                  bool aIsContextMenuKey, int16_t aButton,
+                                  uint16_t aInputSource,
+                                  WinPointerInfo* aPointerInfo = nullptr);
   virtual bool DispatchWindowEvent(mozilla::WidgetGUIEvent* aEvent,
                                    nsEventStatus& aStatus);
   void DispatchPendingEvents();
-  bool DispatchPluginEvent(UINT aMessage, WPARAM aWParam, LPARAM aLParam,
-                           bool aDispatchPendingEvents);
+  void DispatchCustomEvent(const nsString& eventName);
 
 #ifdef ACCESSIBILITY
   /**
    * Return an accessible associated with the window.
    */
-  mozilla::a11y::Accessible* GetAccessible();
+  mozilla::a11y::LocalAccessible* GetAccessible();
 #endif  // ACCESSIBILITY
 
   /**
@@ -355,20 +351,13 @@ class nsWindow final : public nsWindowBase {
 
   const IMEContext& DefaultIMC() const { return mDefaultIMC; }
 
-  virtual void SetCandidateWindowForPlugin(
-      const mozilla::widget::CandidateWindowPosition& aPosition) override;
-  virtual void DefaultProcOfPluginEvent(
-      const mozilla::WidgetPluginEvent& aEvent) override;
-  virtual void EnableIMEForPlugin(bool aEnable) override;
-  virtual nsresult OnWindowedPluginKeyEvent(
-      const mozilla::NativeEventData& aKeyEventData,
-      nsIKeyEventInPluginCallback* aCallback) override;
-  void DispatchPluginSettingEvents();
-
   void GetCompositorWidgetInitData(
       mozilla::widget::CompositorWidgetInitData* aInitData) override;
   bool IsTouchWindow() const { return mTouchWindow; }
   bool SynchronouslyRepaintOnResize() override;
+  virtual void MaybeDispatchInitialFocusEvent() override;
+
+  virtual void LocalesChanged() override;
 
  protected:
   virtual ~nsWindow();
@@ -446,11 +435,12 @@ class nsWindow final : public nsWindowBase {
                               LRESULT* aRetValue);
   bool ExternalHandlerProcessMessage(UINT aMessage, WPARAM& aWParam,
                                      LPARAM& aLParam, MSGResult& aResult);
-  bool ProcessMessageForPlugin(MSG aMsg, MSGResult& aResult);
   LRESULT ProcessCharMessage(const MSG& aMsg, bool* aEventDispatched);
   LRESULT ProcessKeyUpMessage(const MSG& aMsg, bool* aEventDispatched);
   LRESULT ProcessKeyDownMessage(const MSG& aMsg, bool* aEventDispatched);
-  static bool EventIsInsideWindow(nsWindow* aWindow);
+  static bool EventIsInsideWindow(
+      nsWindow* aWindow,
+      mozilla::Maybe<POINT> aEventPoint = mozilla::Nothing());
   // Convert nsEventStatus value to a windows boolean
   static bool ConvertStatus(nsEventStatus aStatus);
   static void PostSleepWakeNotification(const bool aIsSleepMode);
@@ -458,8 +448,8 @@ class nsWindow final : public nsWindowBase {
   TimeStamp GetMessageTimeStamp(LONG aEventTime) const;
   static void UpdateFirstEventTime(DWORD aEventTime);
   void FinishLiveResizing(ResizeState aNewState);
-  nsIntPoint GetTouchCoordinates(WPARAM wParam, LPARAM lParam);
-  Maybe<mozilla::PanGestureInput> ConvertTouchToPanGesture(
+  LayoutDeviceIntPoint GetTouchCoordinates(WPARAM wParam, LPARAM lParam);
+  mozilla::Maybe<mozilla::PanGestureInput> ConvertTouchToPanGesture(
       const mozilla::MultiTouchInput& aTouchInput, PTOUCHINPUT aOriginalEvent);
   void DispatchTouchOrPanGestureInput(mozilla::MultiTouchInput& aTouchInput,
                                       PTOUCHINPUT aOSEvent);
@@ -508,8 +498,9 @@ class nsWindow final : public nsWindowBase {
   static void ScheduleHookTimer(HWND aWnd, UINT aMsgId);
   static void RegisterSpecialDropdownHooks();
   static void UnregisterSpecialDropdownHooks();
-  static bool GetPopupsToRollup(nsIRollupListener* aRollupListener,
-                                uint32_t* aPopupsToRollup);
+  static bool GetPopupsToRollup(
+      nsIRollupListener* aRollupListener, uint32_t* aPopupsToRollup,
+      mozilla::Maybe<POINT> aEventPoint = mozilla::Nothing());
   static bool NeedsToHandleNCActivateDelayed(HWND aWnd);
   static bool DealWithPopups(HWND inWnd, UINT inMsg, WPARAM inWParam,
                              LPARAM inLParam, LRESULT* outResult);
@@ -526,6 +517,10 @@ class nsWindow final : public nsWindowBase {
   void UpdateGlass();
   bool WithinDraggableRegion(int32_t clientX, int32_t clientY);
 
+  bool DispatchTouchEventFromWMPointer(UINT msg, LPARAM aLParam,
+                                       const WinPointerInfo& aPointerInfo,
+                                       mozilla::MouseButton aButton);
+
  protected:
 #endif  // MOZ_XUL
 
@@ -589,7 +584,6 @@ class nsWindow final : public nsWindowBase {
   bool mIsTopWidgetWindow;
   bool mInDtor;
   bool mIsVisible;
-  bool mUnicodeWidget;
   bool mPainting;
   bool mTouchWindow;
   bool mDisplayPanFeedback;
@@ -602,6 +596,7 @@ class nsWindow final : public nsWindowBase {
   bool mOpeningAnimationSuppressed;
   bool mAlwaysOnTop;
   bool mIsEarlyBlankWindow;
+  bool mIsShowingPreXULSkeletonUI;
   bool mResizable;
   DWORD_PTR mOldStyle;
   DWORD_PTR mOldExStyle;
@@ -620,13 +615,14 @@ class nsWindow final : public nsWindowBase {
   static nsWindow* sCurrentWindow;
   static BOOL sIsOleInitialized;
   static HCURSOR sHCursor;
-  static imgIContainer* sCursorImgContainer;
+  static Cursor sCurrentCursor;
   static bool sSwitchKeyboardLayout;
   static bool sJustGotDeactivate;
   static bool sJustGotActivate;
   static bool sIsInMouseCapture;
   static bool sHaveInitializedPrefs;
   static bool sIsRestoringSession;
+  static bool sFirstTopLevelWindowCreated;
 
   PlatformCompositorWidgetDelegate* mCompositorWidgetDelegate;
 
@@ -656,7 +652,7 @@ class nsWindow final : public nsWindowBase {
 
   float mAspectRatio;
 
-  nsCOMPtr<nsIIdleServiceInternal> mIdleService;
+  nsCOMPtr<nsIUserIdleServiceInternal> mIdleService;
 
   // Draggable titlebar region maintained by UpdateWindowDraggingRegion
   LayoutDeviceIntRegion mDraggableRegion;
